'use strict';

exports.__esModule = true;
exports.default = findAndReplaceAttrs;
function findAndReplaceAttrs(path, t) {
  var quasis = path.node.quasi.quasis;
  var stubs = path.node.quasi.expressions;
  var didFindAtLeastOneMatch = false;

  var _quasis$reduce = quasis.reduce(function (accum, quasi, i) {
    var str = quasi.value.cooked;
    var regex = /attr\(([\S]+)(?:\s*(em|ex|px|rem|vw|vh|vmin|vmax|mm|cm|in|pt|pc|%)?)(?:,\s*([\S^)]+))?\)/gm;
    var attrMatch = void 0;
    var matches = [];
    while ((attrMatch = regex.exec(str)) !== null) {
      didFindAtLeastOneMatch = true;
      matches.push({
        value: attrMatch[0],
        propName: attrMatch[1],
        valueType: attrMatch[2],
        defaultValue: attrMatch[3],
        index: attrMatch.index
      });
    }
    var cursor = 0;

    var _loop = function _loop(j) {
      var match = matches[j];
      var value = match.value;
      var propName = match.propName;
      var valueType = match.valueType;
      var defaultValue = match.defaultValue;
      var index = match.index;

      var preAttr = '' + str.slice(cursor, index);
      cursor = index + value.length;
      var postAttr = '' + str.slice(cursor);

      if (preAttr) {
        accum[0].push(t.templateElement({ raw: preAttr, cooked: preAttr }, false));
      }

      if (postAttr && j === matches.length - 1) {
        accum[0].push(t.templateElement({ raw: postAttr, cooked: postAttr }, i === quasis.length - 1));
      }

      var createMemberExpression = function createMemberExpression() {
        return t.memberExpression(t.identifier('props'), t.identifier(propName));
      };

      var returnValue = createMemberExpression();

      if (valueType) {
        returnValue = t.binaryExpression('+', createMemberExpression(), t.stringLiteral(valueType));
      }

      if (defaultValue) {
        returnValue = t.conditionalExpression(createMemberExpression(), createMemberExpression(), t.parenthesizedExpression(t.binaryExpression('+', t.stringLiteral(defaultValue), t.stringLiteral(valueType || ''))));
      }

      var body = t.blockStatement([t.returnStatement(returnValue)]);

      var expr = t.functionExpression(t.identifier('get' + (propName.charAt(0).toUpperCase() + propName.slice(1))), [t.identifier('props')], body);
      accum[1].push(expr);
    };

    for (var j = 0; j < matches.length; ++j) {
      _loop(j);
    }

    if (matches.length === 0) {
      accum[0].push(quasi);
      if (stubs[i]) {
        accum[1].push(stubs[i]);
      }
    } else if (stubs[i]) {
      accum[1].push(stubs[i]);
    }

    return accum;
  }, [[], []]),
      nextQuasis = _quasis$reduce[0],
      nextStubs = _quasis$reduce[1];

  if (didFindAtLeastOneMatch) {
    return t.templateLiteral(nextQuasis, nextStubs);
  }

  return path.node.quasi;
}