'use strict';

exports.__esModule = true;
exports.inserted = exports.sheet = undefined;
exports.flush = flush;
exports.css = css;
exports.fragment = fragment;
exports.keyframes = keyframes;
exports.fontFace = fontFace;
exports.hydrate = hydrate;

var _sheet = require('./sheet');

var _hash = require('./hash');

var _hash2 = _interopRequireDefault(_hash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var sheet = exports.sheet = new _sheet.StyleSheet();
sheet.inject();

var inserted = exports.inserted = {};

function values(cls, vars) {
  var hash = (0, _hash2.default)([cls].concat(vars));
  var varCls = 'vars-' + hash;
  if (inserted[hash]) {
    return varCls;
  }
  var src = vars.map(function (val, i) {
    return '--' + cls + '-' + i + ': ' + val;
  }).join('; ');
  sheet.insert('.' + varCls + ' {' + src + '}');
  inserted[hash] = true;

  return varCls;
}

function flush() {
  sheet.flush();
  exports.inserted = inserted = {};
  sheet.inject();
}

function css(cls, vars, content) {
  if (content) {
    // inline mode
    var src = content.apply(undefined, vars); // returns an array
    var hash = (0, _hash2.default)(src);

    if (!inserted[hash]) {
      inserted[hash] = true;
      src.map(function (r) {
        return r.replace(new RegExp(cls, 'gm'), cls + '-' + hash);
      }).forEach(function (r) {
        return sheet.insert(r);
      });
    }
    return cls + '-' + hash;
  }
  return cls + (vars && vars.length > 0 ? ' ' + values(cls, vars) : '');
}

function fragment(vars, content) {
  return content.apply(undefined, vars);
}

function keyframes(kfm, vars, content) {
  var src = content.apply(undefined, vars);
  var hash = (0, _hash2.default)(src);
  if (!inserted[hash]) {
    inserted[hash] = true;
    src.forEach(function (r) {
      return sheet.insert('@keyframes ' + kfm + '-' + hash + ' {' + r + '}');
    });
  }
  return kfm + '-' + hash;
}

function fontFace(fontRules, vars, content) {
  var src = content.apply(undefined, vars);
  var hash = (0, _hash2.default)(src);
  if (!inserted[hash]) {
    inserted[hash] = true;
    src.forEach(function (r) {
      return sheet.insert(r);
    });
  }
  return fontRules + '-' + hash;
}

function hydrate(ids) {
  ids.forEach(function (id) {
    return inserted[id] = true;
  });
}