'use strict';

exports.__esModule = true;
exports.inline = inline;
exports.keyframes = keyframes;
exports.fontFace = fontFace;

var _parser = require('./parser');

var _hash = require('./hash');

var _hash2 = _interopRequireDefault(_hash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function extractNameFromProperty(str) {
  var regex = /name\s*:\s*([A-Za-z0-9\-_]+)\s*/gm;
  var match = regex.exec(str);
  if (match) {
    return match[1];
  }
}

function getName(extracted, identifierName, prefix) {
  var parts = [];
  parts.push(prefix);
  if (extracted) {
    parts.push(extracted);
  } else if (identifierName) {
    parts.push(identifierName);
  }
  return parts.join('-');
}

function inline(quasi, identifierName, prefix, inlineVars) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash2.default)([].concat(strs) // todo - add current filename?
  );var name = getName(extractNameFromProperty(strs.join('xxx')), identifierName, prefix);
  var hasApply = void 0;
  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== quasi.expressions.length) {
      // todo - test for preceding @apply
      var applyMatch = /@apply\s*$/gm.exec(str);
      if (applyMatch) {
        hasApply = true;
        arr.push('--' + name + '-' + hash + '-' + i);
      } else arr.push('var(--' + name + '-' + hash + '-' + i + ')');
    }
    return arr;
  }, []).join('').trim();

  var rules = (0, _parser.parseCSS)('.' + name + '-' + hash + ' { ' + src + ' }');
  if (hasApply) {
    rules = rules.map(function (rule) {
      return rule.replace(/@apply\s+--[A-Za-z0-9-_]+-([0-9]+)/gm, function (match, p1) {
        return 'xxx' + p1 + 'xxx';
      });
    });
  }
  if (inlineVars || hasApply) {
    rules = rules.map(function (rule) {
      return rule.replace(/var\(--[A-Za-z0-9-_]+-([0-9]+)\)/gm, function (match, p1) {
        return 'xxx' + p1 + 'xxx';
      });
    });
  }

  return { hash: hash, name: name, rules: rules, isStatic: !hasApply && !inlineVars };
}

function keyframes(quasi, identifierName, prefix) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash2.default)([].concat(strs));
  var name = getName(extractNameFromProperty(strs.join('xxx')), identifierName, prefix);

  return {
    hash: hash,
    name: name,
    rules: [(0, _parser.parseCSS)('{ ' + strs.join('').trim() + ' }').join('').trim()]
  };
}

function fontFace(quasi, identifierName, prefix) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash2.default)([].concat(strs));
  var name = getName(extractNameFromProperty(strs.join('xxx')), identifierName, prefix);
  return {
    hash: hash,
    name: name,
    rules: [(0, _parser.parseCSS)('@font-face {' + strs.join('').trim() + '}', {
      nested: false
    }).join('').trim()]
  };
}